using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.ComponentModel;

namespace FenixOPCBarGraph
{
    /// <summary>
    /// Represents one Bar on FenixOPCBarGraph Control
    /// </summary>
    [Serializable]
    public class Bar
    {
        /// <summary>
        /// Direction of current bar
        /// </summary>
        public enum BarDirection 
        {
            /// <summary>Positive direction</summary>
            Positive,
            /// <summary>Negative direction</summary>
            Negative 
        };
        
        private String name = "Bar";
        private BarDirection direction = BarDirection.Positive;
        private double maxValue = 100;
        private double minValue = 0;
        private double barValue = 0;
        private OPCTagConfig tagConfig = new OPCTagConfig();
        private Color barColor = Color.Blue;
        private Color backColor = Color.Black;
        private bool showNumericValue = true;
        private Font barFont = new Font("Times New Roman", 10, FontStyle.Bold);
        private Color fontColor = Color.White;

        #region Constructors

        /// <summary>
        /// Default constructor. 
        /// Reqired for design time bar collection editor
        /// </summary>
        public Bar() {}

        /// <summary>
        /// Creates new Bar with specified name
        /// </summary>
        /// <param name="name">Name of bar</param>
        public Bar(String name)
        {
            this.name = name;
        }
        
        /// <summary>
        /// Creates new Bar with specified name and color
        /// </summary>
        /// <param name="name">Name of bar</param>
        /// <param name="barColor">Bar color</param>
        public Bar(String name, Color barColor)
        {
            this.name = name;
            this.barColor = barColor;
        }

        /// <summary>
        /// Creates new Bar with specified name, color and OPC tag configuration
        /// </summary>
        /// <param name="name">Name of bar</param>
        /// <param name="barColor">Bar color</param>
        /// <param name="tagConfig">OPC tag configuration</param>
        public Bar(String name, Color barColor, OPCTagConfig tagConfig)
        {
            this.name = name;
            this.barColor = barColor;
            this.tagConfig = tagConfig;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Bar direction
        /// </summary>
        /// <remarks>
        /// Use <b>Direction</b> property to specify bar direction.  
        /// This property is set using value from <b>FenixOPCBarGraph.Bar.BarDirection</b> enumeration.
        /// </remarks>
        [Category("Appearance"), Description("Bar direction"), DefaultValue(BarDirection.Positive)]
        public BarDirection Direction
        {
            get { return direction; }
            set { direction = value; }
        }
        
        /// <summary>
        /// Bar name
        /// </summary>
        /// <remarks>
        /// Use the <b>Name</b> property to specify bar name.  
        /// <note>
        /// Multiple bars with same name can coexist on same FenixOPCBarGraph control.
        /// </note>
        /// </remarks>
        [Category("Data"), Description("Bar name"), DefaultValue("Bar")]
        public String Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// Bar color
        /// </summary>
        /// <remarks>
        /// Use <b>BarColor</b> property to specify fill color of rectangle. 
        /// This property is set using one of the values from <b>System.Drawing.Color</b> object.
        /// </remarks>
        [Category("Appearance"), Description("Bar color"), DefaultValue(typeof(Color), "Blue")]
        public Color BarColor
        {
            get { return barColor; }
            set { barColor = value; }
        }

        /// <summary>
        /// Bar background color
        /// </summary>
        /// <remarks>
        /// Use the <b>BackColor</b> property to specify background color of current Bar rectangle. 
        /// This property is set using value from <b>System.Drawing.Color</b> object.
        /// </remarks>
        [Category("Appearance"), Description("Bar background color"), DefaultValue(typeof(Color), "Black")]
        public Color BackColor
        {
            get { return backColor; }
            set { backColor = value; }
        }

        /// <summary>
        /// Shows or hides numeric value on bar
        /// </summary>
        /// <remarks>
        /// Use the <b>ShowNumericValue</b> property to show or hide numeric representation of current Bar value.  
        /// If value of the property is <c>true</c> then numeric value will be displayed.
        /// </remarks>
        [Category("Appearance"), Description("Show/Hide numeric value"), DefaultValue(true)]
        public bool ShowNumericValue
        {
            get { return showNumericValue; }
            set { showNumericValue = value; }
        }

        /// <summary>
        /// Font for numeric value
        /// </summary>
        /// <remarks>
        /// Use the <b>BarFont</b> property to specify font for numeric value. 
        /// This property is set using value from <b>System.Drawing.Font</b> object.
        /// <note>
        /// This has influence only if NumericValue property is <c>true</c>.
        /// </note>
        /// </remarks>
        [Category("Appearance"), Description("Progress number font"), DefaultValue(typeof(Font), "Times New Roman, 10pt, style=Bold")]
        public Font BarFont
        {
            get { return barFont; }
            set { barFont = value; }
        }

        /// <summary>
        /// Color of numeric value text
        /// </summary>
        /// <remarks>
        /// Use the <b>FontColor</b> property to specify color of numeric value font.  
        /// This property is set using one of the values from <b>System.Drawing.Color</b> object.
        /// <note>
        /// This has influence only if NumericValue property is <c>true</c>.
        /// </note>
        /// </remarks>
        [Category("Appearance"), Description("Progress number color"), DefaultValue(typeof(Color), "White")]
        public Color FontColor
        {
            get { return fontColor; }
            set { fontColor = value; }
        }

        /// <summary>
        /// Maximum positive Bar value
        /// </summary>
        /// <remarks>
        /// Use the <b>MaxValue</b> property to specify maximum value for source (OPC Tag). 
        /// <notes>
        /// If not set correctly there will be problems with bar scaling.
        /// </notes>
        /// </remarks>
        [Category("Data"), Description("Maximum Bar value"), DefaultValue(100.0)]
        public double MaxValue
        {
            get { return maxValue; }
            set 
            { 
                if(value > minValue) maxValue = value;
                else throw new ArgumentException("The value must be greater than MinValue.");
            }
        }

        /// <summary>
        /// Minimum Bar value
        /// </summary>
        /// <remarks>
        /// Use the <b>MinValue</b> property to specify minimum value for source (OPC Tag). 
        /// <notes>
        /// If not set correctly there will be problems with scaling.
        /// </notes>
        /// </remarks>
        [Category("Data"), Description("Minimum Bar value"), DefaultValue(0.0)]
        public double MinValue
        {
            get { return minValue; }
            set 
            { 
                if(value < maxValue) minValue = value;
                else throw new ArgumentException("The value must be less than MaxValue.");
            }
        }

        /// <summary>
        /// Bar value
        /// </summary>
        /// <remarks>
        /// Use the <b>Value</b> property to set value of Bar. 
        /// <notes>
        /// This property extends Bar functionality, because, this way, Bar can be used even if there is no OPC Tag assigned.
        /// </notes>
        /// </remarks>
        [Category("Data"), Description("Bar value"), DefaultValue(0.0)]
        public double Value
        {
            get { return barValue; }
            set { barValue = value; }
        }

        /// <summary>
        /// OPC Tag configuration
        /// </summary>
        /// <remarks>
        /// Use the <b>TagConfig</b> property to assign OPC Tag to Bar. 
        /// This property is set using value from <b>FenixOPCBarGraph.OPCTagConfig</b> object.
        /// </remarks>
        [Category("Data"), Description("OPC Tag Configuration")]
        [Editor(typeof(OPCTagEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
        public OPCTagConfig TagConfig
        {
            get { return tagConfig; }
            set { tagConfig = value; }
        }

        private bool ShouldSerializeTagConfig()
        {
            if (tagConfig.TagName != null) return true;
            return false;
        }

        #endregion

        /// <summary>
        /// Convert bar object to string
        /// </summary>
        /// <remarks>
        /// This method just overrides default functionality of <c>ToString()</c> method.
        /// <note>
        /// Be careful if overriding this method because this can have unwanted consequences for BarCollectionEditor functionality.
        /// </note>
        /// </remarks>
        /// <returns>Name of bar or "Bar" if name isn't set</returns>
        public override string ToString()
        {
            if (name != "") return name;
            return "Bar";
        }
    }
}
