using System;
using System.Collections.Generic;
using System.Text;
using System.Data.Odbc;

namespace FenixOPCBarGraph
{
    /// <summary>
    /// Data Logging configuration
    /// </summary>
    [Serializable]
    public class DataLoggingConfig
    {
        private String connString = null;
        private String table = null;
        private String barGraphIDColumn = null;
        private String barIDColumn = null;
        private String tagValueColumn = null;
        private String timestampColumn = null;
        private String barGraphID = null;


        #region Properties

        /// <summary>
        /// ODBC connection string
        /// </summary>
        /// <remarks>
        /// Contains ODBC connection string.
        /// <example>
        /// <code>
        /// // Sets connection string for "testdb" DSN with username "scott" and password "tiger"
        /// DataLoggingConfig dlc = new DataLoggingConfig;
        /// dlc.connString = "DSN=testdb; Uid=scott; Pwd=tiger";
        /// </code>
        /// </example>
        /// </remarks>
        public String ConnString
        {
            get { return connString; }
            set { connString = value; }
        }

        /// <summary>
        /// Database table
        /// </summary>
        /// <remarks>
        /// Name of a table in database which will be used for Data Logging. 
        /// </remarks>
        public String Table
        {
            get { return table; }
            set { table = value; }
        }

        /// <summary>
        /// Table column for FenixOPCBarGraph control identification
        /// </summary>
        /// <remarks>
        /// Name of column in table that will be used to uniquely identify current BarGraph control.
        /// <note>
        /// Data type of this column should be <i>Varchar</i> or <i>Text</i>.
        /// </note>
        /// </remarks>
        public String BarGraphIDColumn
        {
            get { return barGraphIDColumn; }
            set { barGraphIDColumn = value; }
        }

        /// <summary>
        /// Table column for FenixOPCBarGraph.Bar identification
        /// </summary>
        /// <remarks>
        /// Name of column in table that will be used to uniquely identify one Bar on BarGraph control. 
        /// <note>
        /// Data type of this column should be <i>Varchar</i> or <i>Text</i>.
        /// </note>
        /// </remarks>
        public String BarIDColumn
        {
            get { return barIDColumn; }
            set { barIDColumn = value; }
        }

        /// <summary>
        /// Table column for logging OPC Tag value
        /// </summary>
        /// <remarks>
        /// Name of column in table that will be used to store Bar values. 
        /// <note>
        /// Data type of this column should be <i>Number(with double precision)</i>.
        /// </note>
        /// </remarks>
        public String TagValueColumn
        {
            get { return tagValueColumn; }
            set { tagValueColumn = value; }
        }

        /// <summary>
        /// Table column for logging OPC Tag timestamp value
        /// </summary>
        /// <remarks>
        /// Name of column in table that will be used to store timestamps of Bar values.
        /// <note>
        /// Data type of this column should be <i>DataTime</i> or <i>Timestamp</i>.
        /// </note>
        /// </remarks>
        public String TimestampColumn
        {
            get { return timestampColumn; }
            set { timestampColumn = value; }
        }

        /// <summary>
        /// Field for unique BarGraph control identification
        /// </summary>
        /// <remarks>
        /// Value of this field represents name of FenixOPCBarGraph Control.
        /// </remarks>
        public String BarGraphID
        {
            get { return barGraphID; }
            set { barGraphID = value; }
        }

        #endregion

        /// <summary>
        /// Creates SQL INSERT string
        /// </summary>
        /// <param name="barID">Bar ID</param>
        /// <param name="tagValue">Tag value</param>
        /// <param name="timestamp">Timestamp value</param>
        /// <returns>"INSERT" string for SQL database </returns>
        public String CreateInsertString(String barID, Double tagValue, DateTime timestamp)
        {           
            return string.Format("INSERT INTO {0}({1}, {2}, {3}, {4}) VALUES('{5}', '{6}', {7}, {{ts '{8}'}})", 
                                  table,
                                  barGraphIDColumn,
                                  barIDColumn,
                                  tagValueColumn,
                                  timestampColumn,
                                  barGraphID, 
                                  barID,
                                  tagValue,
                                  timestamp.ToString("yyyy-MM-dd HH:mm:ss"));
        }
        
        /// <summary>
        /// Override for display in property browser
        /// </summary>
        /// <remarks>
        /// This method just overrides default functionality of <c>ToString()</c> method.
        /// <note>
        /// Be careful if overriding this method because this can have unwanted consequences for Property Browser functionality.
        /// </note>
        /// </remarks>
        /// <returns>"(Configuration)"</returns>
        public override string ToString()
        {
            return "(Configuration)";
        }
    }
}
