using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Data.Odbc;


namespace FenixOPCBarGraph
{
    /// <summary>
    /// Main OPCBarGraph Control class
    /// </summary>
    public partial class FenixOPCBarGraph : UserControl
    {
        /// <summary>
        /// Creates new FenixOPCBarGraph Control 
        /// </summary>
        public FenixOPCBarGraph()
        {
            InitializeComponent();

            strFormat.Alignment = StringAlignment.Center;
            strFormat.LineAlignment = StringAlignment.Center;
        }

        /// <summary>
        /// Orientations for bars on control
        /// </summary>
        public enum BarOrientation 
        { 
            /// <summary>Horizonal orientation</summary>
            Horizontal, 
            /// <summary>Vertical orientation</summary>
            Vertical 
        };
        
        private bool preview = false;
        private List<Bar> bars = new List<Bar>();
        private int barSpacing = 0;
        private BarOrientation orientation = BarOrientation.Vertical;
        private StringFormat strFormat = new StringFormat();
        private DataLoggingConfig loggingConfig = new DataLoggingConfig();
        private bool loggingEnabled = false;
        
        [NonSerialized]
        private OdbcConnection connection = null;

        #region Properties
        
        /// <summary>
        /// Bar collection
        /// </summary>
        /// <remarks>
        /// Use the <b>Bars</b> property to specify bar objects. 
        /// This property is set using collection of <b>FenixOPCBarGraph.Bar</b> objects.
        /// </remarks>
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
        [Editor(typeof(BarCollectionEditor), typeof(System.Drawing.Design.UITypeEditor)), Category("Data")]
        [Description("Bar collection")]
        public List<Bar> Bars
        {
            get { return bars; }
            set { bars = value; }
        }

        private bool ShouldSerializeBars()
        {
            if (bars.Count > 0) return true;
            return false;
        }

        /// <summary>
        /// Orientation for all bars on control
        /// </summary>
        /// <remarks>
        /// Use the <b>Orientation</b> property to specify orientation for all bars on control. 
        /// This property is set using one of values in <b>FenixOPCBarGraph.BarOrientation</b> enumeration.
        /// </remarks>
        [Category("Appearance"), Description("Bar orientation"), DefaultValue(BarOrientation.Vertical)]
        public BarOrientation Orientation
        {
            get { return orientation; }
            set { orientation = value; }
        }

        /// <summary>
        /// Refresh interval for all bars
        /// </summary>
        /// <remarks>
        /// Use the <b>RefreshInterval</b> property to specify refresh interval for all bars. 
        /// Unit for this property value is <i>milisecond</i>.
        /// </remarks>
        [Category("Data"), Description("Refresh interval for bars"), DefaultValue(100)]
        [TypeConverter(typeof(RefreshIntervalConverter))]
        public int RefreshInterval
        {
            get { return dataRefreshTimer.Interval; }
            set { dataRefreshTimer.Interval = value; }
        }

        /// <summary>
        /// Real-time preview in Design mode
        /// </summary>
        /// <remarks>
        /// Use the <b>Preview</b> property to see visual preview in Design mode. 
        /// If this property is <c>true</c> control will perform all run-time functionality in design mode.
        /// <note>
        /// This property should be used only inside Visual Studio Designer. 
        /// For third-party IDEs this property should be ignored.
        /// </note>
        /// </remarks>
        [Category("Data"), Description("Real time preview in Design mode"), DefaultValue(false), DesignOnly(true)]
        public bool Preview
        {
            get { return preview; }
            set { preview = value; }
        }

        /// <summary>
        /// Space between bars
        /// </summary>
        /// <remarks>
        /// Use the <b>BarSpacing</b> property to specify space between bars. 
        /// Unit for this property value is <i>pixel</i>.
        /// </remarks>
        [Category("Data"), Description("Space between bars"), DefaultValue(0)]
        [TypeConverter(typeof(BarSpacingConverter))]
        public int BarSpacing
        {
            get { return barSpacing; }
            set { if (value >= 0) barSpacing = value; }
        }

        /// <summary>
        /// DataLogging Configuration
        /// </summary>
        /// <remarks>
        /// Use the <b>LoggingConfig</b> property to specify DataLogging configuration. 
        /// This property is set using <b>FenixOPCBarGraph.DataLoggingConfig</b> object.
        /// </remarks>
        [Category("DataLogging"), Description("DataLogging configuration")]
        [Editor(typeof(DataLoggingEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public DataLoggingConfig LoggingConfig
        {
            get { return loggingConfig; }
            set { loggingConfig = value;}
        }

        private bool ShouldSerializeLoggingConfig()
        {
            if (loggingConfig.ConnString != null) return true;
            return false;
        }

        /// <summary>
        /// Enable/Disable DataLogging
        /// </summary>
        /// <remarks>
        /// Use the <b>LoggingEnabled</b> property to enable or disable Data Logging. 
        /// If value of this property is <c>true</c> Data Logging will be enabled.
        /// </remarks>
        [Category("DataLogging"), Description("Enable/Disable DataLogging"), DefaultValue(false)]
        public bool LoggingEnabled
        {
            get { return loggingEnabled; }
            set 
            {
                if (loggingConfig.ConnString != null)
                {
                    loggingEnabled = value;
                    if (value == true)
                    {
                        try
                        {
                            connection = new OdbcConnection(loggingConfig.ConnString);
                            connection.Open();
                        }
                        catch (Exception ex)
                        {
                            loggingEnabled = false;
                            MessageBox.Show("Database Error: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else connection.Close();
                }
                else throw new ArgumentException("You must configure DataLogging!");
            }
        }

        #endregion

        #region Drawing functions

        private void Draw(Graphics g)
        {
            int offset = 0;
            int barWidth = 0;

            foreach (Bar bar in bars)
            {
                Brush frontBrush = new SolidBrush(bar.BarColor);
                Brush backBrush = new SolidBrush(bar.BackColor);

                Rectangle backRect;
                Rectangle frontRect;

                Double valueRange = Math.Abs(bar.MaxValue - bar.MinValue);

                if (orientation == BarOrientation.Vertical)
                {
                    barWidth = (this.Width - barSpacing * (bars.Count - 1)) / bars.Count;
                    backRect = new Rectangle(offset, 0, barWidth, this.Height);

                    if (bar.Direction == Bar.BarDirection.Positive)
                    {
                        frontRect = new Rectangle(offset, this.Height - (int)((bar.Value - bar.MinValue) * this.Height / valueRange), barWidth, this.Height);
                    }
                    else
                    {
                        frontRect = new Rectangle(offset, 0, barWidth, (int)((bar.Value - bar.MinValue) * this.Height / valueRange));
                    }
                }
                else
                {
                    barWidth = (this.Height - barSpacing * (bars.Count - 1)) / bars.Count;
                    backRect = new Rectangle(0, offset, this.Width, barWidth);

                    if (bar.Direction == Bar.BarDirection.Positive)
                    {
                        frontRect = new Rectangle(0, offset, (int)((bar.Value - bar.MinValue) * this.Width / valueRange), barWidth);
                    }
                    else
                    {
                        frontRect = new Rectangle(this.Width - (int)((bar.Value - bar.MinValue) * this.Width / valueRange), offset, this.Width, barWidth);
                    }
                }

                g.FillRectangle(backBrush, backRect);
                g.FillRectangle(frontBrush, frontRect);

                if (bar.ShowNumericValue)
                {
                    g.DrawString(Math.Round(bar.Value, 3).ToString(),
                                 bar.BarFont,
                                 new SolidBrush(bar.FontColor),
                                 backRect,
                                 strFormat);
                }

                offset += barSpacing + barWidth;
            }
        }

        private void FenixOPCBarGraph_Paint(object sender, PaintEventArgs e)
        {
            if (this.DesignMode && !preview) return;
            Draw(e.Graphics);
        }

        private void FenixOPCBarGraph_Resize(object sender, EventArgs e)
        {
            if (this.DesignMode && !preview) return;
            this.Invalidate();
        }

        #endregion

        private void dataRefreshTimer_Tick(object sender, EventArgs e)
        {
            if (this.DesignMode && !preview) return;

            foreach (Bar bar in bars)
            {
                if ((bar.TagConfig != null) && (bar.TagConfig.TagName != null))
                {
                    try
                    {
                        bar.Value = Convert.ToDouble(opcSystemsComponent.GetTag_Parameter_Value("Value",
                                                                                                "Value",
                                                                                                bar.TagConfig.TagName,
                                                                                                bar.TagConfig.Group,
                                                                                                bar.TagConfig.Host));

                        // Data Logging
                        if (loggingEnabled)
                        {
                            DateTime timestamp = Convert.ToDateTime(opcSystemsComponent.GetTag_Parameter_Value("Value",
                                                                                                       "TimeStamp",
                                                                                                        bar.TagConfig.TagName,
                                                                                                        bar.TagConfig.Group,
                                                                                                        bar.TagConfig.Host));

                            OdbcCommand sqlCommand = new OdbcCommand(loggingConfig.CreateInsertString(bar.Name, bar.Value, timestamp), connection);
                            sqlCommand.ExecuteNonQuery();
                        }
                    }
                    catch (Exception ex)
                    {
                        this.LoggingEnabled = false;
                        MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }

            this.Invalidate();
        }

        #region API

        /// <summary>
        /// Gets bar by Name. 
        /// </summary>
        /// <param name="name">Bar name</param>
        /// <returns>First found bar in collection</returns>
        /// <example>
        /// <code>
        /// // Gets two bars from existing OPCBarGraph control
        /// Bar temparatureBar = myOPCBarGraphControl.getBarByName("Temperature");
        /// Bar preassureBar = myOPCBarGraphControl.getBarByName("Preassure");
        /// </code>
        /// </example>

        public Bar getBarByName(String name)
        {
            foreach (Bar b in bars)
            {
                if (b.Name == name) return b;
            }

            return null;
        }

        #endregion
    }
}
